#!/usr/bin/perl
#
# $Id: la-larrd.pl,v 1.10 2003/03/18 16:48:12 cncook Exp $
#
# la-larrd.pl - larrd module for collecting Load Average stats 
#
# Scott Walters Copyright 1999
# scott@PacketPushers.com 
#
# Christian Pearce
# pearcec@PacketPushers.com
#  
# Version 1.1   Craig Cook   29 Sept 2001
#            Added cacheflow cpu check
#         1.2   Craig Cook   1 Aug 2002
#            Allow for DISPREALLOADAVG=TRUE
#            Use 5 min value if available
#
# Do whatever you want with it
# just don't sell it and make money off of it and not cut me in ;)
# I take no liability for it, use at your own risks

# Set a path to the rrd tool lib
# Set as and $ENV and see what is up
use lib $ENV{'RRDPM'};
use RRDs;

$ds = "cpu";

@ds = ("DS:la:GAUGE:600:0:U",
       );

##################################
# Start of script
##################################

if( not $ENV{'BBHOME'} ) {
        print "$0: BBHOME is not set\n";
        exit(1);
}

if( not -d $ENV{'BBHOME'} ) {
        print "$0: BBHOME is invalid\n";
        exit(1);
}

if( not -d $ENV{'BBRRDS'} ) {
        print "$0: BBRRDS is not set\n";
        exit(1);
}

##
# real work begins here
##

# go to the logs area for simplication of the list
chdir "$ENV{'BBLOGS'}";

# Glob the la logs
LOG: while (  defined ($log = <*.$ds>)  ) {

        $host=$log;
        # create some handy vars
        $host =~ s/\.$ds//;
        $host =~ s/\,/\./g;

        ($ctime) = (stat($log))[10];

        $colour_check="notok";

	# Grab Load Avg line
        open(LOG,"$log");
        while ( defined ($line = <LOG>) ) {
	  chomp $line;

          # ensure colour is valid - ie don't graph purple results
	  if($line =~ /green|yellow|red/) {
  	    # New attempt and grabing procs and users.
	    $line =~ /(\d+)\s+users,\s+(\d+)\s+procs/o;
	    $users=$1;
	    $procs=$2;
            $colour_check="ok";

	    if($line =~ /%/) {
	      # This might break NT, but hope it didn't lose
	      # anything in the translation of Andreas Holzhammer fix
	      $line =~ s/[^%]*[=]([0-9][0-9]*)%.*/$1/;
              # We only get here if we have a valid cpu %
	      $la=$line;
  	    } else {
		($dummy,$line) = split /=/,$line,3;	
              # We only get here if we have a valid cpu %
                if ($line =~ /\./) { # ie DISPREALLOADAVG=TRUE
                  $line = $line * 100 ;
                }
	        $la=$line;
	    } # end if($line =~ /%/)

          } elsif ($line =~ /CPU usage/) {
              $line =~ s/CPU usage:\s+(\d+)%[^,]+,\s*(\d+)%/$1,$2/;
              # We only get here if we have a valid cpu %
              # Use the 5min avg if possible, else use what we find
              $la_1min=$1;
              $la_5min=$2;
              if ( defined $la_5min ) {
                   $la=$la_5min;
                  } else {
                      $la=$la_1min;
                  }
          } # end line colour check

          if ($colour_check ne "ok") {
		print "$0: WARN $host is not showing red, yellow, or green. $log skipped.\n" if ($ENV{'WARN'});
		next LOG;
	  }

        } # end while	
	close(LOG);

	print "$0: DEBUG users: $users, procs: $procs, la: $la\n" if $ENV{'DEBUG'};
	
	# RRD for la
	
	$RRD="$ENV{'BBRRDS'}/$host.la.rrd";

	# if it doesn't exist, make it 
	if ( not -f $RRD ) {
			# RRAS defined in larrd.pl
			@rras = split " ",$ENV{'RRAS'};
			RRDs::create($RRD,@ds,@rras);
                        $ERR=RRDs::error;
                        if($ERR) {
                                print "$0: ERROR creating $RRD: $ERR\n" if ($ENV{'ERROR'});
				next;
                        }
		$ctime="N";
		print "$0: STATUS did not find $RRD, created.\n" if ($ENV{'STATUS'});
	}

	RRDs::update("$RRD","$ctime:$la");
        $ERR=RRDs::error;
        if($ERR) {
 	       print "$0: WARN updating $RRD: $ERR\n" if ($ENV{'WARN'});
        }

	# Sloppy
        ($ctime) = (stat($log))[10];

	# RRD for users

#
# Users
# 

      if (defined $users) {
	$RRD="$ENV{'BBRRDS'}/$host.users.rrd";

	# if it doesn't exist, make it 
	if ( not -f $RRD ) {
			# RRAS defined in larrd.pl
			@rras = split " ",$ENV{'RRAS'};
			RRDs::create($RRD,@ds,@rras);
                        $ERR=RRDs::error;
                        if($ERR) {
                                print "$0: ERROR creating $RRD: $ERR\n" if ($ENV{'ERROR'});
				next;
                        }
		$ctime="N";
		print "$0: STATUS did not find $RRD, created.\n" if ($ENV{'STATUS'});
	}

	RRDs::update("$RRD","$ctime:$users");
        $ERR=RRDs::error;
        if($ERR) {
 	       print "$0: WARN updating $RRD: $ERR\n" if ($ENV{'WARN'});
        }

     } # end if (defined $users)

     #
     # Procs
     # 
      if (defined $procs) {

	# Sloppy
        ($ctime) = (stat($log))[10];

	# RRD for procs
	
	$RRD="$ENV{'BBRRDS'}/$host.procs.rrd";

	# if it doesn't exist, make it 
	if ( not -f $RRD ) {
			# RRAS defined in larrd.pl
			@rras = split " ",$ENV{'RRAS'};
			RRDs::create($RRD,@ds,@rras);
                        $ERR=RRDs::error;
                        if($ERR) {
                                print "$0: ERROR creating $RRD: $ERR\n" if ($ENV{'ERROR'});
				next;
                        }
		$ctime="N";
		print "$0: STATUS did not find $RRD, created.\n" if ($ENV{'STATUS'});
	}

	RRDs::update("$RRD","$ctime:$procs");
        $ERR=RRDs::error;
        if($ERR) {
 	       print "$0: WARN updating $RRD: $ERR\n" if ($ENV{'WARN'});
        }
 
      } # end if (defined $procs) {

}
##############################################
# end of script
##############################################
