#!/usr/bin/perl -w

# $Id: larrd-html.pl,v 1.28 2003/03/18 16:48:12 cncook Exp $

# larrd-html.pl:  BB ext script that creates the html snippet for the BB larrd
# column.

# Scott Walters Copyright 1999
# scott@PacketPushers.com 
#
# Christian Pearce
# pearcec@PacketPushers.com
#  
# 13 Mar 2003  Craig Cook
#              Added apache array
#              Changed  $ENV{'DEBUG'}; to  $DEBUG;
#
# Do whatever you want with it
# just don't sell it and make money off of it and not cut me in ;)
# I take no liability for it, use at your own risks

use strict;
use vars qw/@LARRDDISABLE @DATALIST/;
 
use lib $ENV{'LARRDHOME'};
require "larrd-config.pl";

# Set a path to the rrd tool lib
# Set as and $ENV and see what is up
use lib $ENV{'RRDPM'};
use RRDs;

my $DEBUG = 1 if $ENV{'DEBUG'};

my  @vmstat_array  = qw(vmstat vmstat0 vmstat2 vmstat3 vmstat6 vmstat7 vmstat8 vmstat9);
my  @netstat_array = qw(netstat1 netstat2);
my  @disk_array    = qw(disk disk1);
my  @apache_array  = qw(apache apache1 apache2 apache3);
my  $filesystems_per_graph = 5;

##################################
# Start of script
##################################

if( not $ENV{'BBHOME'} ) {
        print "$0: BBHOME is not set\n";
        exit(1);
}

if( not -d $ENV{'BBHOME'} ) {
        print "$0: BBHOME is invalid\n";
        exit(1);
}

if( not -d $ENV{'BBRRDS'} ) {
        print "$0: BBRRDS is not set\n";
        exit(1);
}

##
# real work begins here
##

my %hh;
my %dh;
 
my $html_d="$ENV{'BBHOME'}/www/html";
my %disabled = map { $_ => 1 } @LARRDDISABLE;  # Disable Hosts Hash

# Dirty hack eventually I would like to see uniform naming with bb and larrd
foreach (@DATALIST) { $_ = "tcp" if $_ eq "bbnet"; }

my %services = map { $_ => 1 } @DATALIST;      # Services hash

opendir(DIR, $ENV{'BBRRDS'}) or die "Could not open BBLOGS $ENV{'BBRRDS'} : $!";
my @rrds = grep { ! /^\./ } readdir DIR;
closedir(DIR);

# we build a string with the services
my $regexp_service = join "|", @DATALIST;

foreach my $rrd (@rrds) {
    # we find $host and $service in $rrd with the previously made regexp
    # string :
    my ($host, $service) = $rrd =~ /^(.*)\.($regexp_service).*rrd$/o;

    next unless $host && $service;

    $host =~ s/\,/\./g; # Stay backwards comp with memory.

    # we skip if $host is disable
    next if defined $disabled{$host};

    # we skip if $service isn't trendable by larrd
    next unless defined $services{$service};

    # Inclusion Jason Kruse -- Add items to disk hash
    if ($service eq "disk") {
      my @items = split /\.disk,/, $rrd;
      next if ! defined $items[1];
      my $disk_name = "," . $items[1];
      substr($disk_name, -4) = "";
      $disk_name = "/" if ($disk_name eq ",root");
      $disk_name =~ s/\,/\//g;
      push @{ $dh{$host} }, $disk_name;
    }

    $hh{$host}{$service}++;
}

my $date = localtime;

my $cl_date = `date`;
chomp $cl_date;

# generate the larrd column based on %hh (host hash)

foreach my $host (keys %hh) {

    print "$0 : DEBUG $host : " if ($DEBUG);

    # Make the bb log to circumvent clean_string in bbd

    my $newhost = $host;
    $newhost =~ s/\./\,/g;

    my $larrd_file = "$ENV{'BBLOGS'}/$newhost.$ENV{'LARRDCOLNAME'}";

    open(LARRD,">$larrd_file");
    print LARRD "green $cl_date - larrd is accumulating ";
    print LARRD "<center><BR>";

#    foreach my $service (sort keys %{$hh{$host}}) {
#  The reason we don't use the line above it to keep the order of the @DATALIST

    foreach my $service (@DATALIST) {
        next unless defined $hh{$host}{$service};

        if ($service eq "vmstat") {
          foreach my $vmstat_element (@vmstat_array) {
            print "$0 : DEBUG $service ($hh{$host}{$service})\n" if ($DEBUG);

            print LARRD qq{<p><A HREF="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$vmstat_element"><IMG SRC="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$vmstat_element&graph=hourly" ALT="larrd is accumulating $vmstat_element" BORDER=0></A>};
          } # end foreach

        } elsif ($service eq "netstat") {
            foreach my $netstat_element (@netstat_array) {
              print "$0 : DEBUG $service ($hh{$host}{$service})\n" if ($DEBUG);

              print LARRD qq{<p><A HREF="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$netstat_element"><IMG SRC="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$netstat_element&graph=hourly" ALT="larrd is accumulating $netstat_element" BORDER=0></A>};
            } # end foreach

        } elsif ($service eq "apache") {
            foreach my $apache_element (@apache_array) {
              print "$0 : DEBUG $service ($hh{$host}{$service})\n" if ($DEBUG);

              print LARRD qq{<p><A HREF="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$apache_element"><IMG SRC="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$apache_element&graph=hourly" ALT="larrd is accumulating $apache_element" BORDER=0></A>};
            } # end foreach

        } elsif ($service eq "disk") {
            foreach my $disk_element (@disk_array) {
              my @sub_array = sort @{ $dh{$host} };
              my $count = 0;
              my $end = 0;
              while (@sub_array) {
                splice(@sub_array, 0, 6);
                $end = $count + $filesystems_per_graph;
                print LARRD qq{<p><A HREF="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$disk_element&disk_part=$count..$end"><IMG SRC="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$disk_element&graph=hourly&disk_part=$count..$end" ALT="larrd is accumulating $service" BORDER=0></A>};
                $count = $end + 1;
              }

            } # end foreach

        } else {
            print "$0 : DEBUG $service ($hh{$host}{$service})\n" if ($DEBUG);

            print LARRD qq{<p><A HREF="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$service"><IMG SRC="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$service&graph=hourly" ALT="larrd is accumulating $service" BORDER=0></A>};
        } # end if ($service eq "vmstat")

	# " NOTE this is not needed if($hh{$host}{$service});
    } # end foreach my $service (@DATALIST)

    print LARRD "</center>";

    close(LARRD);

    # Time stamp it for the future
    my $time = time;
    utime $time, $time + 60 * $ENV{PURPLEDELAY}, $larrd_file;

    if($ENV{'BBLOGSTATUS'} eq "STATIC") {
        $ENV{'QUERY_STRING'}="HOSTSVC=$newhost.$ENV{'LARRDCOLNAME'}";
        my $hostsvc = join "", `$ENV{'LARRDHOSTSVC'}`;
        unless ($hostsvc) {
            $hostsvc=q{Error: You are using Big Brother BBLOGSTATUS as STATIC.  This requires that you set the variable $ENV{'LARRDHOSTSVC'} in larrd-config.pl.  The program defined in $ENV{'LARRDHOSTSVC'}="} . $ENV{'LARRDHOSTSVC'} . q{" returned no output.  Please see your larrd-config.pl file or change BBLOGSTATUS to DYNAMIC.  This is what is recommened.};
        }
        
        $hostsvc =~ s|Content-type: text/html||;
	open (HTML, ">$ENV{'BBHOME'}/www/html/$host.$ENV{'LARRDCOLNAME'}.html")
	  || die "Unable to open $ENV{'BBHOME'}/www/html/$host.$ENV{'LARRDCOLNAME'}.html\n";
	print HTML $hostsvc;
	close HTML;
    }
}
