#!/usr/bin/perl
#
# A script to use rrdtool to check the average load
# on a system over the past 12 hours.
#
# Author: Bill Benedetto <bbenedetto@goodyear.com>
# Version: 1.2 - Jun 14, 2002
#
# 1.3  Craig Cook    3 Mar 2003
#      Modified to work with larrd 0.43x
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You can find the GNU General Public License at
#  http://www.gnu.org/copyleft/gpl.html
# or you can write to the
#  Free Software Foundation, Inc.
#  59 Temple Place - Suite 330
#  Boston, MA 02111-1307, USA.
#
# This program is used in tandem with Big Brother (http://bb4.com/)
# and RRDtool (http://ee-staff.ethz.ch/~oetiker/webtools/rrdtool/).
#
# I run this once a day via crontab.
#
# This program gets the load average for each of my .la.rrd's for the
# past 12 hours and calculates the average over that 12 hours.
# If it's above some threshold (IE, the load average jumped up and
# stayed up), then I flag that as a possible problem.  The script
# makes a list of these systems on stdout.  This list includes URLs
# for the larrd graphs that fit "load average above threshold for past
# 12 hours".
#

use strict;
use Sys::Hostname;

my $host = "myhost";

my $DEBUG = 0;

my $cgidir = "cgi-bin"

my %inputdir = (
                'myhost' => '/usr/local/bbvar/rrd'
                );
my %maxvalues = (
                 'myhost' => 1.0
                 );
                                        # Allow maximum to be specified on the
                                        # command line
my $max_average = shift || $maxvalues{$host};

                                        # Get all the files in the $dir that
                                        # end in "la.${period}.png" or
                                        # "diskpct.${period}.png"...
opendir(THEDIR,$inputdir{$host})
  or die "Couldn't open directory $inputdir{$host}: $!\n";
my @files = grep(/.*la.rrd/,readdir(THEDIR));
closedir(THEDIR);

print "Load averages for past 12 hours greater than ", $max_average, "\n",
      "=================================================\n";

                                        # ... and print out a record for each
foreach my $file (sort @files) {
  open(RRD, "/usr/local/rrdtool/bin/rrdtool fetch $inputdir{$host}/$file AVERAGE -s now-12h|") or
  die "Unable to open rrdtool fetch command for $_: $!\n";

  my $numrec = 0;
  my $total  = 0;
  while(<RRD>) {

    next if /nan/i;                     # Skip "Not a Number" records
    next if /^$/;                       # Skip blank lines

    $numrec++;
    $total += (split)[1];
    print "Number of records = " , $numrec, " Total = ", $total, "\n"
      if $DEBUG;
  }
                                        # Strip out the machine name
  (my $mach = $file) =~ s/(.*)\.la\.rrd/\1/;

  my $average = ($total/100)/$numrec;
  if ($average > $max_average) {
    printf("Average for %s\tis %.1f", $mach, $average);
    print " -> http://$host/$cgidir/bb-hostsvc.sh?HOSTSVC=$mach.larrd\n";
  } 

  if ($DEBUG) {
    printf("Average for %s\tis %.1f", $mach, $average);
    print " -> http://$host/$cgidir/bb-hostsvc.sh?HOSTSVC=$mach.larrd\n";
  }
    
  close(RRD);
}
