#!/usr/bin/perl
#
# nmailq-larrd.pl - larrd module for collecting Mail Queue stats
#
# Do whatever you want with it, just don't sell it!
# I take no liability for it, use at your own risk!

# Set a path to the rrd tool lib
# Set as and $ENV and see what is up

# V1.0 19 Apr 2002    Alan Stellpflug
#
#  1.1 17 Jan 2003    Craig Cook
#      Major rewrite
#      - Added DATA functionality
#      - Uses sub procedures
#      - Handles single and multiple queues
#      - Upgraded bf-nmailq.sh
#      - Added debug output
#
#  1.2 13 Mar 2003    Craig Cook
#      - Changed $ENV{'DEBUG'} to $DEBUG

use lib $ENV{'RRDPM'};
use RRDs;

$ds = "nmailq";

my @ds = ();

$DEBUG = 1 if $ENV{'DEBUG'};

if ($DEBUG) {
  use Data::Dumper ;  # only for debug output
}


##################################
# Start of script
##################################

if( not $ENV{'BBHOME'} ) {
        print "$0: BBHOME is not set\n";
        exit(1);
}

if( not -d $ENV{'BBHOME'} ) {
        print "$0: BBHOME is invalid\n";
        exit(1);
}

if( not -d $ENV{'BBRRDS'} ) {
        print "$0: BBRRDS is not set\n";
        exit(1);
}

my %all_queues = ();
my $qsize = 0;

##
# real work begins here
##

&read_logs;

print Data::Dumper->Dump([\%h_fn],["h_fn"]) if $DEBUG ;

while ( ($host,$log) = each %h_fn ) {

    %all_queues = ();
    $qsize = 0;

    print "$0 : DEBUG host=$host, log=$log\n" if ($DEBUG);

    # 2. Grab the data from the log how ever you need
    &get_info_from_log;

    # Now lets process the list

    &process_log;

    # 3. grab the timestamp for rrd.
   &get_log_time;


  # 4. create the rrd filename

  &update_rrd;

  # delete data log file after we have used it
  unlink $log if $log =~ /$ENV{'BBVAR'}\/data/io;

}

sub read_logs {

  # go to the logs area for simplication of the list
  # Perl Cookbook Recipe 9.5
  opendir(DIR, "$ENV{'BBLOGS'}") or die "$0: $!";
  while ( defined($log = readdir(DIR))) {
        next unless $log =~ /\.$ds$/io;
        my $host = $log;
        $host =~ s/\.$ds//;
        $host =~ s/\,/\./g;

        $h_fn{$host}="$ENV{'BBLOGS'}/$log";
  }
  closedir(DIR);

  opendir(DIR, "$ENV{'BBVAR'}/data") or die "$0: $!";
  while ( defined($log = readdir(DIR))) {
        next unless $log =~ /\.$ds$/io;
        my $host = $log;
        $host =~ s/\.$ds//;
        $host =~ s/\,/\./g;

        $h_fn{$host}="$ENV{'BBVAR'}/data/$log";
  }

  closedir(DIR);

}

sub get_info_from_log {

    print "$0 : DEBUG log=$log\n" if ($DEBUG);

    open(LOG,"$log");

    # Slurp up the log into an array
    @log = <LOG>;

    close(LOG);

}


sub process_log {

  print "$0: DEBUG in process_log\n" if $DEBUG;

  print Data::Dumper->Dump([\%all_queues],["all_queues"]) if $DEBUG ;

  foreach $line (@log) {

    last if ($line =~ /^(purple)/);

    print "$0: DEBUG line: $line\n" if $DEBUG;

    # A single queue will look like this:
    #/var/spool/mqueue (6 requests)
    # or this:
    #/var/spool/mqueue (1 request)
    # or this:
    #/var/spool/mqueue empty

    # Multiple Queues will look something like this
    #  Total requests: 298
    #   /var/spool/mqueue/q1 (61 requests)
    #   /var/spool/mqueue/q2 (50 requests)

    if ($line =~ /\((\d+) request/) {
      $qsize = $1;
    } elsif ($line =~ / empty/) {
      $qsize = 0;
    } else {
      next;
    }
 
    # Assume the last piece of the path is the queue name.

    @line = split " ", $line;
    @path = split "/", $line[0];
    $qname = $path[$#path];

    print "$0: DEBUG Queue: $qname, Size: $qsize\n" if $DEBUG;

    $all_queues{$qname}=$qsize;

  }
}

sub get_log_time {
  # Sometimes you get it from stat, sometimes you get it from the log
  ($ctime) = (stat($log))[10];
  print "$0 : DEBUG ctime = $ctime\n" if ($DEBUG);
}

sub update_rrd {

  $RRD="$ENV{'BBRRDS'}/$host.$ds.rrd";

  print "$0 : RRD=$RRD\n" if ($DEBUG);

  print Data::Dumper->Dump([\%all_queues],["all_queues"]) if $DEBUG ;

  # if it doesn't exist, make it
  if ( not -f $RRD ) {
    &create_rrd;
  }

  $rrdupdate = $ctime;
  foreach my $queue (keys %all_queues) {
    print "$0 : DEBUG updating: $queue\n" if ($DEBUG);
    $rrdupdate .= ":$all_queues{$queue}";
  }

  print "$0 : DEBUG final update: $rrdupdate\n" if ($DEBUG);

  RRDs::update("$RRD","$rrdupdate");
  $ERR=RRDs::error;
  if($ERR) {
    print "$0: WARN updating $RRD: $ERR\n" if ($ENV{'WARN'});
  }
}

sub create_rrd {

  print "$0: DEBUG in create_rrd\n" if $DEBUG;

  print Data::Dumper->Dump([\%all_queues],["all_queues"]) if $DEBUG ;

    foreach my $queue (keys %all_queues) {
      print "$0 : DEBUG creating: $queue\n" if ($DEBUG);
      push @ds, "DS:$queue:GAUGE:600:0:U";
    }

    # RRAS defined in larrd.pl
    @rras = split " ",$ENV{'RRAS'};

    RRDs::create($RRD,@ds,@rras);
    $ERR=RRDs::error;
    if($ERR) {
      print "$0: ERROR creating $RRD: $ERR\n" if ($ENV{'ERROR'});
      next;
    }

    $ctime="N";
    print "$0: STATUS did not find $RRD, created.\n" if ($ENV{'STATUS'});
}

##############################################
# end of script
##############################################
