#!/bin/sh

# Knobbled version of "la-graph.sh" to report Swap
# Chris Warburton. Feb 2000

# set -x

# swap-graph.sh

# this script will build swap usage PNG's
# using $RRDTOOL graph with RRD's built via
# the BB ext script larrd.sh

if test ! "$BBHOME"
then
        echo "$0: BBHOME is not set"
        exit 1
fi

if test ! -d "$BBHOME"
then
        echo "$0: BBHOME is invalid"
        exit 1
fi

# source the conf
. $BBHOME/etc/bbdef.sh
. $BBHOME/ext/larrd/larrd.conf

# get current date
DATE=`date`

# let's keep the graphs seperate from the RRD's
if [ ! -d $GRAPHS ]
then
	mkdir $GRAPHS	
	echo "$0 did not find $GRAPHS, created." 
fi

# get to work
cd $BBRRDS

for rrd in `ls *swap.rrd`
do
	# need some vars again
	HOST=`echo $rrd | $SED 's/\\.swap\.rrd//' `

	# define the graphs for each time period
	HOURLY=$GRAPHS/$HOST.swap.hourly.png
	DAILY=$GRAPHS/$HOST.swap.daily.png
	WEEKLY=$GRAPHS/$HOST.swap.weekly.png
	MONTHLY=$GRAPHS/$HOST.swap.monthly.png

	# make the hourly graph
	$RRDTOOL graph $HOURLY -s e-48h \
		--title "$HOST Swap Usage Last 48 Hours" \
		-z -w 576 -v "Swap Usage (%)" -a PNG  \
		DEF:avg=$rrd:swap:AVERAGE AREA:avg#$COLOR:"5 Minute Average" \
		GPRINT:avg:LAST:'Current\: %5.1lf%S' \
		GPRINT:avg:MIN:'Min\: %5.1lf%S' \
		GPRINT:avg:AVERAGE:'Average\: %5.1lf%S' \
		GPRINT:avg:MAX:'Max\: %5.1lf%S\j' \
		COMMENT:"$DATE\l"


	# make the daily graph
	$RRDTOOL graph $DAILY -s e-12d \
		--title "$HOST Swap Usage Last 12 Days" \
		-w 576 -z -a PNG -v "Swap Usage (%)" \
		DEF:avg=$rrd:swap:AVERAGE AREA:avg#$COLOR:"30 Minute Average" \
                GPRINT:avg:LAST:'Current\: %5.1lf%S' \
                GPRINT:avg:MIN:'Min\: %5.1lf%S' \
                GPRINT:avg:AVERAGE:'Average\: %5.1lf%S' \
                GPRINT:avg:MAX:'Max\: %5.1lf%S\j' \
		COMMENT:"$DATE\l"

	# make the weekly graph
	$RRDTOOL graph $WEEKLY -s e-48d \
		--title "$HOST Swap Usage Last 48 Days" \
		-w 576 -z -a PNG -v "Swap Usage (%)" DEF:avg=$rrd:swap:AVERAGE \
		AREA:avg#$COLOR:"2 Hour Average" \
		GPRINT:avg:LAST:'Current\: %5.1lf%S' \
                GPRINT:avg:MIN:'Min\: %5.1lf%S' \
                GPRINT:avg:AVERAGE:'Average\: %5.1lf%S' \
                GPRINT:avg:MAX:'Max\: %5.1lf%S\j' \
		COMMENT:"$DATE\l"

	# make the monthly graph
	$RRDTOOL graph $MONTHLY -s e-576d \
		--title "$HOST Swap Usage Last 576 Days" \
		-w 576 -z -a PNG -v "Swap Usage (%)" DEF:avg=$rrd:swap:AVERAGE \
		AREA:avg#$COLOR:"1 Day Average" \
		GPRINT:avg:LAST:'Current\: %5.1lf%S' \
                GPRINT:avg:MIN:'Min\: %5.1lf%S' \
                GPRINT:avg:AVERAGE:'Average\: %5.1lf%S' \
                GPRINT:avg:MAX:'Max\: %5.1lf%S\j' \
		COMMENT:"$DATE\l"	

done
