#!/usr/bin/perl
# Change the line above to match the location of the perl interpreter,
# (but leave the #! bit). If you don't know, ask your sys admin.
#
#########################################################################
# larrd-graphs.cgi
# Simple cgi to allow graphing of selected larrd graphs
#
# Ver 1.4 30-Apr-2003 Tom Schmidt tschmidt@micron.com
# See README file for installation instructions and revision history.
#
# Usual rules apply -
# 1) Use at own risk
# 2) Feel free to copy, distribute, modify and so on - but I would appreciate
#    some feedback and ideas/samples of mods
#
# Revision History:
# V1.4 - 30-Apr-2003 Tom Schmidt tschmidt@micron.com
#        Improved sort of hostnames. Enhanced for butter-1.0
# V1.3 - 28-Apr-2003 Tom Schmidt tschmidt@micron.com
#        Improved sort of hostnames
# V1.2 - 27-Feb-2003 Tom Schmidt tschmidt@micron.com
#        Made compatible with larrd-0.43 disk graphs
# V1.1 - 10-Dec-2002 Tom Schmidt <tschmidt@micron.com>
#        Improved setup, output, and diagnostics.
# V1.0 - 04-Dec-2002 Tom Schmidt <tschmidt@micron.com>
#        Created based on en-dis.cgi, larrd-grapher.cgi and larrd-html.pl
#        scripts and released to public.
#########################################################################
#
# Set these paths for your environment:
#
use strict;

$ENV{'BBHOME'} = "/opt/bb";	# Set BBHOME for your site
use lib "/opt/larrd";		# Matches LARRDHOME for your site

# Uncomment the appropriate line below depending on if your larrd version
# supports the "disk_part" parameter or not
#my $has_disk_part = 0;	# larrd 0.42 and earlier, and butter-1.0 (larrd-0.44)
my $has_disk_part = 1;	# larrd 0.43

# Comment the line below if you are using larrd 0.43 instead of butter-1.0
require "larrd-config.pl";

# Uncomment the line below if you are using butter-1.0 or larrd-0.44 instead of
# larrd-0.43 or earlier
#use LARRD::config qw( @DATALIST );

# list of vmstat larrd graphs for your platform
my @vmstat_array  = qw(vmstat vmstat0 vmstat2 vmstat3 vmstat6 vmstat7 vmstat8 vmstat9);

# list of netstat larrd graphs for your platform
my @netstat_array = qw(netstat netstat1 netstat2);

# list of disk larrd graphs for your platform
my @disk_array = qw(disk disk1);
#my @disk_array = qw(disk);
my $filesystems_per_graph = 5;	# Only used if $has_disk_part = 1 above

#$ENV{'DEBUG'} = 1;	# Uncomment for debugging, or may be in larrd config

#
# No more changes should be needed from here down
# Host names are collected automagically, as are services being monitored
#########################################################################

if (not -d $ENV{'LARRDTMP'}) {
	mkdir "$ENV{'LARRDTMP'}",0755;
}
open(TMP,">>$ENV{'LARRDTMP'}/larrd-graphs.debug") if $ENV{'DEBUG'};

use vars qw/@LARRDDISABLE @DATALIST @hosts $host $serv @graphs $graph %rqpairs %dh %hh $request $requestERR/;

# Read bbdef.sh and bbinc.sh for some environment variables
my $cgibinurl_in="";
my $bbweb_in="";
my $fqdn_in;
open(BBDEF,"<$ENV{'BBHOME'}/etc/bbdef.sh") or file_error("$ENV{'BBHOME'}/etc/bbdef.sh","$!");
while (<BBDEF>) {
	$cgibinurl_in=$_ if /^CGIBINURL=/;
	$bbweb_in=$_ if /^BBWEB=/ && $bbweb_in eq "" ;
	$fqdn_in=$_ if /^FQDN=/;
}
close BBDEF;
(undef,$ENV{'CGIBINURL'},undef) = split(/\"/,$cgibinurl_in);
env_error("CGIBINURL","$ENV{'BBHOME'}/etc/bbdef.sh") if $ENV{'CGIBINURL'} eq "";
(undef,$ENV{'BBWEB'},undef) = split(/\"/,$bbweb_in);
env_error("BBWEB","$ENV{'BBHOME'}/etc/bbdef.sh") if $ENV{'BBWEB'} eq "";
(undef,my $fqdn,undef) = split(/\"/,$fqdn_in);	# Extract FQDN Definition
my $bbskin_in="";
open(BBINC,"grep \"^BBSKIN=\" $ENV{'BBHOME'}/etc/bbinc.sh |") or file_error("$ENV{'BBHOME'}/etc/bbinc.sh","$!");
while (<BBINC>) {
	$bbskin_in=$_;
}
close BBINC;
(undef,$ENV{'BBSKIN'},undef) = split(/\"/,$bbskin_in);
$ENV{'BBSKIN'} =~ s/\$\{BBWEB\}/$ENV{'BBWEB'}/g;
env_error("BBSKIN","$ENV{'BBHOME'}/etc/bbinc.sh") if $ENV{'BBSKIN'} eq "";

# eventually I would like to see uniform naming with bb and larrd
foreach (@DATALIST) {
	$_ = "tcp" if $_ eq "bbnet";
}
my %disabled = map { $_ => 1 } @LARRDDISABLE;  # Disable Hosts Hash
my %services = map { $_ => 1 } @DATALIST;      # Services hash

opendir(DIR, $ENV{'BBRRDS'}) or file_error("$ENV{'BBRRDS'}","$!");
my @rrds = grep { ! /^\./ } readdir DIR;
closedir(DIR);

# we build a string with the services
my $regexp_service = join "|", @DATALIST;

foreach my $rrd (@rrds) {
    # we find $host and $service in $rrd with the previously made regexp
    # string :
    my ($host, $service) = $rrd =~ /^(.*)\.($regexp_service).*rrd$/o;

    next unless $host && $service;

    $host =~ s/\,/\./g; # Stay backwards comp with memory.

    # we skip if $host is disable
    next if defined $disabled{$host};

    # we skip if $service isn't trendable by larrd
    next unless defined $services{$service};

    # Inclusion Jason Kruse -- Add items to disk hash
    if ($service eq "disk") {
      my @items = split /\.disk/, $rrd;
      next if ! defined $items[1];
      my $disk_name = $items[1];
      $disk_name =~ s/\.rrd//g;
      $disk_name =~ s/\,/\//g;
      $disk_name = "/" if ($disk_name eq "/root");
      push @{ $dh{$host} }, $disk_name;
#      print TMP "hosts hash:$disk_name {$service}\n";
    }

    $hh{$host}{$service}++;
#    print TMP "hosts hash:$hh{$host}{$service}\n";
}

my %donehost;
open(HOSTS,"<$ENV{'BBHOME'}/etc/bb-hosts") or file_error("$ENV{'BBHOME'}/etc/bb-hosts","$!");
while (<HOSTS>) {
	next if /^#/ or /^\s/ or ! /^[0-9]/;
	(undef,$host)=split(/\s+/);
	# Strip domain name unless fully qualified domain names are used
	($host,undef)=split(/\./,$host) if ($fqdn ne "TRUE");
	push @hosts,$host if not $donehost{$host};
	$donehost{$host}=1;
}
close HOSTS;

my @services = @DATALIST;

@graphs = qw(hourly daily weekly monthly);

print TMP "hosts list:\n@hosts\n";
print TMP "services list:\n@services\n";

&get_request;
print "Content-type: text/html\n\n";

if ($rqpairs{'submit'} eq "GO" && $requestERR eq "" ) {
	file_insert("bbrep_header", "green");
	print "<CENTER>\n";
	print "<table>\n";
	foreach $serv(@services) {
		if ($rqpairs{$serv} eq "2") {
			if ($serv eq "vmstat") {
				&graphit(@vmstat_array);
			} elsif ($serv eq "netstat") {
				&graphit(@netstat_array);
			} elsif ($serv eq "disk") {
				&graphit(@disk_array);
			} else {
				&graphit($serv);
			}
		}
	}
	print "</table><br>\n";
	print "larrd is proud to be built on ";
	print "<A HREF=\"http://ee-staff.ethz.ch/~oetiker/webtools/rrdtool/\">";
	print "RRDtool</A>\n";
} else {
	file_insert("bbrep_header", "blue");
	my @my_name_is = split(/\//,$0);
	my $selected = "";
	print "<CENTER>\n";
	if ($request ne "" && $requestERR ne "") {
		print "<FONT COLOR=\"#FF0000\">\n";
		print "<H3>\n$requestERR</H3>\n</FONT>\n";
	}
	print "<form action=$my_name_is[$#my_name_is] method=post>";
	print "<table width=80% border=1>";
	print "<tr><td valign=top><font size=+1 face=\"helvetica, sans-serif\">1) Select one or more hosts</font></td>";
	print "<td valign=top><font size=+1 face=\"helvetica, sans-serif\">2) Select larrd graph(s) for selected host(s)</font></td>";
	print "<tr><td rowspan=8 valign=top>";
	my $line;
	print "<table width=100%>";
	print "<tr><td width=100%><select multiple name=\"hosts\" size=25>\n";
	foreach $line(sort alphanumerically @hosts) {
		# write request line so select options work for multiple choices
		if ($rqpairs{$line} eq "1") {
			$selected = " selected";
		} else {
			$selected = "";
		}
		print "<option$selected value=\"&amp;$line=1\">$line\n";
	}
	print "</select></td></tr>\n";
	print "</table>";

	print "</td></tr>
		\n";

	print "<tr><td valign=top><table width=100% border=0>\n";
	print "<tr><td width=100%><select multiple name=\"serv\" size=12>\n";
	foreach $line(@services) {
		# write request line so select options work for multiple choices
		if ($rqpairs{$line} eq "2") {
			$selected = " selected";
		} else {
			$selected = "";
		}
		print "<option$selected value=\"&amp;$line=2\">$line\n";
	}
	print "</select></td></tr>\n";
	print "</table></td></tr>\n";

	print "<tr><td valign=top><font size=+1 face=\"helvetica, sans-serif\">3) Select graph period(s)</font></td></tr>";
	print "<tr><td valign=top><table width=100% border=0>\n";
	print "<tr><td width=100%><select multiple name=\"graph\" size=4>\n";
	foreach $line(@graphs) {
		# write request line so select options work for multiple choices
		if ($rqpairs{$line} eq "3") {
			$selected = " selected";
		} else {
			$selected = "";
		}
		print "<option$selected value=\"&amp;$line=3\">$line\n";
	}
	print "</select></td></tr>\n";
	print "</table></td></tr>\n";

	print "<tr><td><font size=+1 face=\"helvetica, sans-serif\">4) Press <input name=submit type=submit value=GO></font></td></tr>\n";
	print "</table></form>";
}
print "<FONT COLOR=\"#FF0000\">\n<H6><I>";
print "&copy; Copyright Quest Software, Inc.  1997-2002  All rights reserved.";
print "</I></H6>\n</FONT></CENTER>\n";
print "</BODY></HTML>\n";
exit;

sub get_request {
	my $content_length = $ENV{'CONTENT_LENGTH'};
	read( STDIN, $request, $content_length );
	# Rewrite request line so select options work for multiple choices.
	# Assumes host names and service names don't have "&" or "=" in them.
	$request =~ s/%26/&/g;	# Change %26 to "&"
	$request =~ s/%3D/=/g;	# Change %3D to "="
	print TMP "request:\n$request\n";
    	%rqpairs = &url_decode(split(/[&=]/, $request));
	# Check request to make sure it has sufficient arguments
	$requestERR = "";
	$requestERR .= "Missing host selection<br>" if ($request !~ /=1/);
	$requestERR .= "Missing larrd graph selection<br>" if ($request !~ /=2/);
	$requestERR .= "Missing graph period selection<br>" if ($request !~ /=3/);
}

sub url_decode {
    	foreach (@_)
	{
		tr/+/ /;
		s/%(..)/pack("c",hex($1))/ge;
    	}
    	@_;
}

sub graphit {
    my $primary_serv = @_[0];
    foreach $serv(@_) {
        foreach $graph(@graphs) {
            if ($rqpairs{$graph} eq "3") {
                foreach $host(sort alphanumerically keys %hh) {
                    if ($rqpairs{$host} eq "1") {
                        print "<tr><th>$graph $serv trends for $host</th></tr>\n";
                        if (defined $hh{$host}{$primary_serv}) {
                            if ($primary_serv eq "disk") {
                                if ($has_disk_part eq 0) {
                                    print "<tr><td><IMG SRC=\"$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$serv&graph=$graph\" ALT=\"larrd is accumulating\"><BR></td></tr>\n";
                                } else {
                                    my @sub_array = sort alphanumerically @{ $dh{$host} };
                                    my $count = 0;
                                    my $end = 0;
                                    while (@sub_array) {
                                        splice(@sub_array, 0, 6);
                                        $end = $count + $filesystems_per_graph;
                                        print "<tr><td><IMG SRC=\"$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$serv&graph=$graph&disk_part=$count..$end\" ALT=\"larrd is accumulating\"><BR></td></tr>\n";
                                        $count = $end + 1;
                                    }
                                }
                            } else {
                                print "<tr><td><IMG SRC=\"$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$serv&graph=$graph\" ALT=\"larrd is accumulating\"><BR></td></tr>\n";
                            }
                        } else {
                            print "<tr><td align=center>No data to graph<br><br></td></tr>\n";
                        }
                    }
                }
            }
        }
    }
}

sub file_insert {
	my ($file, $bkg_color) = (shift, shift);
	open (F, "$ENV{'BBHOME'}/web/$file") || file_error($file) && return;
	local $/ = undef;
	my $content = <F>;
	close F;
	my $date = localtime();
	for ($content) {
		s/&BBDATE/$date/g;
		s/&BBSKIN/$ENV{'BBSKIN'}/g;
		s/&BBWEB/$ENV{'BBWEB'}/g;
		s/&BBBACKGROUND/$bkg_color/g;
		s/[Aa]vailability [Rr]eport/larrd graphs/g;
		s/Click on the colored icon for details//g;
	}
	print $content;
	print "\n";
}

sub file_error {
	my $file = shift;
	print <<EOF;
Content-type: text/html

<HTML><BODY>
<HR size=4>
<BR>$ENV{'BBHOME'}/web/$file is either missing or invalid, please create this file with your custom header<BR><BR>
<HR size=4>
EOF
}

sub env_error {
	my $variable = shift;
	my $file = shift;
	print <<EOF;
Content-type: text/html

<HTML><BODY>
<HR size=4>
<BR>Could not find environment variable $variable in $file<BR>
Please correct your Big Brother environment.<BR><BR>
<HR size=4>
EOF
	exit 1;
}

sub file_error {
	my $file = shift;
	my $error = shift;
	print <<EOF;
Content-type: text/html

<HTML><BODY>
<HR size=4>
<BR>$file : $error<BR>
Please correct your Big Brother environment.<BR><BR>
<HR size=4>
EOF
	die "$file : $error";
}

sub alphanumerically {
	local($&, $`, $', $1, $2, $3, $4);
	# Sort numbers numerically
	return $a cmp $b if $a !~ /\D/ && $b !~ /\D/;
	# Handle things like Level1, ..., Level10
	if ($a =~ /^(.*\D)(\d+)$/) {
		my($a1, $a2) = ($1, $2);
		if ($b =~ /^(.*\D)(\d+)$/) {
			my($b1, $b2) = ($1, $2);
			return $a2 <=> $b2 if $a1 eq $b1;
		}
	}
	# Default is to sort alphabetically
	return $a cmp $b;
}

